// Tab.cc for Fluxbox
// Copyright (c) 2001 Henrik Kinnunen (fluxgen@linuxmail.org)
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.

#include "Tab.hh"
#include <iostream>
#ifdef HAVE_CONFIG_H
#  include "../config.h"
#endif // HAVE_CONFIG_H
#include "i18n.hh"

unsigned int Tab::size_w = 64;
unsigned int Tab::size_h = 16;

Tab::Tab(FluxboxWindow *win, Tab *prev, Tab *next) { 
	m_prev = prev; m_next = next; 
	m_win = win;
	m_display = Fluxbox::instance()->getXDisplay();
	createTabWindow();
}

Tab::~Tab() {
	//link the list
	if (m_prev)
		m_prev->m_next = m_next;
	if (m_next)
		m_next->m_prev = m_prev;

	Fluxbox::instance()->removeTabSearch(m_tabwin);			//remove this from list
	XDestroyWindow(m_display, m_tabwin);				//destroy window
}


//createTabWindow
//creates the Window for tab to be above the title window
void Tab::createTabWindow() {
	//create window
	unsigned long attrib_mask = CWBackPixmap | CWBackPixel | CWBorderPixel |
			      CWColormap | CWOverrideRedirect | CWEventMask;
  XSetWindowAttributes attrib;
  attrib.background_pixmap = None;
  attrib.background_pixel = attrib.border_pixel =
			    m_win->screen->getBorderColor()->getPixel();
  attrib.colormap = m_win->screen->getColormap();
  attrib.override_redirect = True;
  attrib.event_mask = ButtonPressMask | ButtonReleaseMask |
                      ButtonMotionMask | ExposureMask;
	m_tabwin = XCreateWindow(m_display, m_win->screen->getRootWindow(), 
										m_win->frame.x, m_win->frame.y-size_h, 
										size_w, size_h,
									0, m_win->screen->getDepth(),
                  InputOutput, m_win->screen->getVisual(), attrib_mask, &attrib);
	//set grab
	XGrabButton(m_display, Button1, Mod1Mask, m_tabwin, True,
				ButtonReleaseMask | ButtonMotionMask, GrabModeAsync,
				GrabModeAsync, None, Fluxbox::instance()->getMoveCursor());
				
	//save to tabsearch
	Fluxbox::instance()->saveTabSearch(m_tabwin, this);	
	XMoveResizeWindow(m_display, m_tabwin,
									 m_win->frame.x, m_win->frame.y-size_h,
									 size_w, size_h);	
	XMapSubwindows(m_display, m_tabwin);
	XMapWindow(m_display, m_tabwin);
}

void Tab::raise() {
	//raise tab window
	m_win->screen->raiseWindows(&m_tabwin, 1);
}

//draw
//draws the tab 
void Tab::draw(bool pressed) {
	
	if (m_win->focused) {
		if (m_win->frame.flabel)
      XSetWindowBackgroundPixmap(m_display, m_tabwin, m_win->frame.flabel);
    else
      XSetWindowBackground(m_display, m_tabwin, m_win->frame.flabel_pixel);			
	} else {
		if (m_win->frame.ulabel)
      XSetWindowBackgroundPixmap(m_display, m_tabwin, m_win->frame.ulabel);
    else
      XSetWindowBackground(m_display, m_tabwin, m_win->frame.ulabel_pixel);
	}
	//clear the background to right color/pixmap
	XClearWindow(m_display, m_tabwin);
	
	//Draw title string	

	unsigned int l = m_win->client.title_text_w;
	int dlen=m_win->client.title_len;
	int dx=m_win->frame.bevel_w*2;
	
	if (m_win->client.title_text_w > size_w) {
		for (; dlen >= 0; dlen--) {
			if (I18n::instance()->multibyte()) {
				XRectangle ink, logical;
				XmbTextExtents(m_win->screen->getWindowStyle()->fontset, 
											m_win->client.title, dlen,
											&ink, &logical);
				l = logical.width;
			} else
				l = XTextWidth(m_win->screen->getWindowStyle()->font, 
									m_win->client.title, dlen);
				l += (m_win->frame.bevel_w * 4);

			if (l < size_w)
				break;
		}
	}
	
	switch (m_win->screen->getWindowStyle()->justify) {
	case BScreen::RightJustify:
		dx += size_w - l;
		break;

	case BScreen::CenterJustify:
		dx += (size_w - l) / 2;
		break;
	}
	
	//Draw title to m_tabwin
	
	if (I18n::instance()->multibyte()) {
		XmbDrawString(m_display, m_tabwin,
			m_win->screen->getWindowStyle()->fontset,
			((m_win->focused) ? m_win->screen->getWindowStyle()->l_text_focus_gc :
		   m_win->screen->getWindowStyle()->l_text_unfocus_gc), dx, 1 -
		  m_win->screen->getWindowStyle()->fontset_extents->max_ink_extent.y,
			m_win->client.title, dlen);
	} else {
		XDrawString(m_display, m_tabwin,
			((m_win->focused) ? m_win->screen->getWindowStyle()->l_text_focus_gc :
			 m_win->screen->getWindowStyle()->l_text_unfocus_gc), dx,
			m_win->screen->getWindowStyle()->font->ascent + 1, 
			m_win-> client.title, dlen);
	}
}

//setPosition
//position tab after m_win ( follow the window pos )
void Tab::setPosition() {	
	XMoveWindow(m_display, m_tabwin, m_win->frame.x, m_win->frame.y-size_h);	
}

//Event handlers
///////////////////////////////////

//buttonPressEvent
//handle button press event here
void Tab::buttonPressEvent(XButtonEvent *be) {	
	#ifdef DEBUG
	cerr<<__FILE__<<","<<__LINE__<<": Tab pressed"<<endl;
	#endif
	draw(true);
	//set window to titlewindow so we can take advatage of drag function
	be->window = m_win->frame.title;
	//call windows buttonpress eventhandler
	m_win->buttonPressEvent(be);
}

//buttonReleaseEvent
//handle button release event here
void Tab::buttonReleaseEvent(XButtonEvent *be) {		
	//raise this
	raise();
	//set window to title window soo we can use m_win handler for menu
	be->window = m_win->frame.title;
	//call windows buttonrelease event handler
	m_win->buttonReleaseEvent(be);
}

//exposeEvent
//handle expose event here
void Tab::exposeEvent(XExposeEvent *ee) {
	draw(false);
}

